#!/bin/bash
# pleace .fastq or .gz raw data in the fastq_gz folder.
# please note that sample name will be retrieved from the .fastq file name before the "_" character (example: SAMPLENAME_L001_R1_001.fastq)

echo " " ;
echo "Collecting Raw Data" ;

mkdir temp ;
ls fastq_gz | cut -d " " -f 1 > temp/miseq_folders_temp1.txt ;
sed 's/_/\t/' temp/miseq_folders_temp1.txt | cut -f1 > temp/miseq_folders_temp2.txt ;
sort temp/miseq_folders_temp2.txt | uniq > temp/miseq_folderspre.txt ;
grep -v ".fastq" temp/miseq_folderspre.txt > temp/miseq_folders.txt ;

cd fastq_gz ;
find ./*/ -type f -exec mv '{}' ./ \; &>/dev/null
rm -r */ &>/dev/null ;
cd .. ;

ls fastq_gz/ | cut -d " " -f 1 > temp/name_files.txt ;
sed 's/_/\t/' temp/name_files.txt | cut -f1 > temp/name_samples_temp1.txt ;
sort temp/name_samples_temp1.txt | uniq > temp/name_samples.txt ;
sed "s/$/\t/g" temp/name_samples.txt > sample_list_and_treatment.txt ;


echo " " ;
echo "Creating folders"
mkdir results ;
mkdir results/filterings_report ;
mkdir results/by_sample ;
mkdir results/by_output ;
mkdir results/by_output/taxonomy ;
mkdir results/by_output/OTUs_table ;
mkdir filterings ;
cd fastq_gz ;
gunzip -d *.gz &>/dev/null ;
cd .. ;
ls fastq_gz/ | cut -d " " -f 1 > temp/name_files.txt ;
sed 's/_/\t/' temp/name_files.txt | cut -f1 > temp/name_samples_temp1.txt ;
sort temp/name_samples_temp1.txt | uniq > temp/name_samples.txt ;

#CICLO FILTERINGS AND ANALYSES
while read MKDIRSAMPLES
do
mkdir filterings/$MKDIRSAMPLES ;
mkdir filterings/$MKDIRSAMPLES/temp ;
cp sample_list_and_treatment.txt filterings/$MKDIRSAMPLES/temp/. ;
cp fastq_gz/${MKDIRSAMPLES}_S* filterings/$MKDIRSAMPLES/. ;
cp -R software/ filterings/$MKDIRSAMPLES/. ;
cp -R software/ITS_lactobacilli filterings/$MKDIRSAMPLES/. ;

cp software/ITS_lactobacilli/Fasting_Map_ITS_fw.txt filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp1.txt ;
sed "s/SAMPLENAME/$MKDIRSAMPLES/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp1.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp2.txt ;
sed "s/\t/#\t/g" filterings/$MKDIRSAMPLES/temp/sample_list_and_treatment.txt > filterings/$MKDIRSAMPLES/temp/sample_list_and_treatment_mod.txt ;
grep "$MKDIRSAMPLES#" filterings/$MKDIRSAMPLES/temp/sample_list_and_treatment_mod.txt | cut -f2 > filterings/$MKDIRSAMPLES/temp/treatment.txt ;
sed "s/#//g" filterings/$MKDIRSAMPLES/temp/treatment.txt > filterings/$MKDIRSAMPLES/temp/treatment_mod.txt ;
TREATMENT=$(cat filterings/$MKDIRSAMPLES/temp/treatment_mod.txt) ;
sed "s/TREATMENT/$TREATMENT/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp2.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_3.txt ;
sed "s/\-/minus/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_3.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_4.txt ;
sed "s/\_/underline/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_4.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_5.txt ;
sed "s/\./dot/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_5.txt > filterings/$MKDIRSAMPLES/Fasting_Map_fw.txt ;

cp software/ITS_lactobacilli/Fasting_Map_ITS_rv.txt filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp1.txt ;
sed "s/SAMPLENAME/$MKDIRSAMPLES/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp1.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp2.txt ;
sed "s/TREATMENT/$TREATMENT/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp2.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_3.txt ;
sed "s/\-/minus/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_3.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_4.txt ;
sed "s/\_/underline/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_4.txt > filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_5.txt ;
sed "s/\./dot/g" filterings/$MKDIRSAMPLES/temp/Fasting_Map_temp_5.txt > filterings/$MKDIRSAMPLES/Fasting_Map_rv.txt ;

echo "cd filterings/$MKDIRSAMPLES/ ;" >> exec_ITS.sh ;
echo "echo '   processing sample $MKDIRSAMPLES' ;" >> exec_ITS.sh ;
echo "echo ' ' ;" >> exec_ITS.sh ;
echo "./software/ITS_lactobacilli_FWRV_analysis_02_02_2018.sh ;" >> exec_ITS.sh ;
echo "rm -fr filterings/$MKDIRSAMPLES/software ;" >> exec_ITS.sh ;
echo "cd .. ;" >> exec_ITS.sh ;
echo "cd .. ;" >> exec_ITS.sh ;
done < temp/name_samples.txt
#FINE CICLO

echo " " ;
echo "Analysis of the samples"
echo " " ;
chmod +x exec_ITS.sh ;
./exec_ITS.sh ;
mv exec_ITS.sh temp/exec_ITS.sh ;


echo "Summarizing results"
#CICLO RESULTS
while read MKDIRSAMPLES
do
cp -R filterings/$MKDIRSAMPLES/output results/by_sample/$MKDIRSAMPLES ;
cp filterings/$MKDIRSAMPLES/output/taxonomy_output_blast.txt results/by_output/taxonomy/${MKDIRSAMPLES}_taxonomy.txt ;
cp filterings/$MKDIRSAMPLES/output/taxonomy_output_blast_normalized_by_rRNA_loci_number.txt results/by_output/taxonomy/${MKDIRSAMPLES}_taxonomy_normalized_by_rRNA_loci_number.txt ;

cp software/ITS_paste.sh results/by_output/taxonomy/. ;
cd results/by_output/taxonomy/ ;
./ITS_paste.sh ;
cd .. ;
cd .. ;
cd .. ;

cp filterings/$MKDIRSAMPLES/output/OTUs_table_taxonomy.txt results/by_output/OTUs_table/${MKDIRSAMPLES}_OTUs_table.txt
done < temp/name_samples.txt
#FINE CICLO

echo " " ;
echo "Generating quality-filtering statistics"
#CICLO FILTERING STATS
while read MKDIRSAMPLES
do

grep "number of sequenced pe reads" filterings/$MKDIRSAMPLES/split_library_output/number_of_sequenced_pe_reads.txt >> temp/number_of_sequenced_pe_reads.txt ;
grep "quality filtered forward reads" filterings/$MKDIRSAMPLES/split_library_output/quality_filtered_forward_reads.txt >> temp/quality_filtered_forward_reads.txt ;
grep "quality filtered reverse reads" filterings/$MKDIRSAMPLES/split_library_output/quality_filtered_reverse_reads.txt >> temp/quality_filtered_reverse_reads.txt ;

done < temp/name_samples.txt
#FINE CICLO


sed "s/number of sequenced pe reads //g" temp/number_of_sequenced_pe_reads.txt > temp/number_of_sequenced_pe_reads_ok.txt ;
sed "s/quality filtered forward reads //g" temp/quality_filtered_forward_reads.txt > temp/quality_filtered_forward_reads_ok.txt ;
sed "s/quality filtered reverse reads //g" temp/quality_filtered_reverse_reads.txt > temp/quality_filtered_reverse_reads_ok.txt ;


echo "Sample#Number of sequenced pe reads#Number of quality-filtered forward reads#Number of quality-filtered reverse reads" > temp/first_row.txt ;
sed "s/#/\t/g" temp/first_row.txt > temp/first_row_ok.txt ;
paste temp/name_samples.txt temp/number_of_sequenced_pe_reads_ok.txt temp/quality_filtered_forward_reads_ok.txt temp/quality_filtered_reverse_reads_ok.txt > temp/pasted.txt ;

cat temp/first_row_ok.txt temp/pasted.txt > results/filterings_report/filtering_report.txt ;

### END QIIME ANALYSIS

echo " " ;
echo "Removing temporary files"

mv results/ output ;
rm -fr temp ;
rm -fr filterings ;
rm sample_list_and_treatment.txt ;
rm output/by_output/taxonomy/ITS_paste.sh ;
mv output/by_output/taxonomy/for_excel/vlookup_column.txt /output/by_output/taxonomy/for_excel/list_of_detected_species.txt ;
mv output/by_output/taxonomy/for_excel/taxonomy.txt output/by_output/taxonomy/for_excel/summary_of_normalized_profiles.txt ;
mv output/by_output/taxonomy/for_excel output/by_output/taxonomy/summary_for_excel ;


echo " " ;
echo "End of lactobacilli ITS profiling analysis"
echo " " ;

