#!/bin/bash
############################################################################
#                                                                          #
#                Accepted input formats are .bam and .fastq                #
#                                                                          #
#       The input file must be renamed input.bam or input.fastq and        #
#            located in the same directory as this script file             #
#                                                                          #
#              output will be located in the output directory              #
#                                                                          #
#            Requires Qiime to be installed (or Virtual Machine)           #
#                                                                          #
############################################################################


# Pre-processing

mkdir tree ;
mkdir otus ;
mkdir otus/rep_set ;
mkdir otus/counts ;
mkdir output ;
mkdir temp  &>/dev/null ;
mkdir temp/counts/ ;
echo "" > otus/rep_set/seqs_rep_set.fasta



mkdir temp  &>/dev/null ;
mkdir fastq

mv *.fastq fastq/. ;


software/./fastq-mcf --qual-mean 20 -w 5 -q 20 -k 0 -l 100 n/a fastq/*_L001_R1_001.fastq fastq/*_L001_R2_001.fastq -o fastq/filtered_reads_1.fastq -o fastq/filtered_reads_2.fastq &>/dev/null ;

software/convert_project -f fastq -t fasta fastq/filtered_reads_1.fastq input_1 &>/dev/null ;
software/convert_project -f fastq -t fasta fastq/filtered_reads_2.fastq input_2 &>/dev/null ;

mv input_1.fasta.qual input_1.qual ;
mv input_2.fasta.qual input_2.qual ;

rm -rf split_library_output ; split_libraries.py -b 0 -m Fasting_Map_fw.txt -H 7 -l 100 -L 400 -s 20 -M 5 -f input_1.fasta -q input_1.qual -o split_library_output_1
rm -rf split_library_output ; split_libraries.py -b 0 -m Fasting_Map_rv.txt -H 7 -l 100 -L 400 -s 20 -M 5 -f input_2.fasta -q input_2.qual -o split_library_output_2

mkdir split_library_output ;
cat split_library_output_1/seqs.fna split_library_output_2/seqs.fna > split_library_output/seqs.fna ;


STAT1=$(grep "^" fastq/*_L001_R1_001.fastq | wc -l | awk '{print $1}' ) ;
STAT1BY4=$((STAT1/4)) ;
echo "number of sequenced pe reads $STAT1BY4" > split_library_output/number_of_sequenced_pe_reads.txt ;
STAT2a=$(grep "^" split_library_output_1/seqs.fna | wc -l | awk '{print $1}' ) ;
STAT2aBY2=$((STAT2a/2)) ;
echo "quality filtered forward reads $STAT2aBY2" > split_library_output/quality_filtered_forward_reads.txt ;
STAT2b=$(grep "^" split_library_output_2/seqs.fna | wc -l | awk '{print $1}' ) ;
STAT2bBY2=$((STAT2b/2)) ;
echo "quality filtered reverse reads $STAT2bBY2" > split_library_output/quality_filtered_reverse_reads.txt ;
STAT3=$(grep "^" split_library_output/seqs.fna | wc -l | awk '{print $1}' ) ;
STAT3BY2=$((STAT3/2)) ;
echo "quality filtered reverse reads $STAT3BY2" > split_library_output/quality_filtered_fwrv_reads.txt ;


###OTUs GENERATION AND TAXONOMIC CLASSIFICATION###

#picks denovo otus with sequences identical in the first 300nt.

pick_otus.py -i split_library_output/seqs.fna -o otus/DeNovo_otus/ -m prefix_suffix --prefix_length 300 ;

pick_rep_set.py -i otus/DeNovo_otus/seqs_otus.txt -f split_library_output/seqs.fna -l otus/rep_set/seqs_rep_set.log -o otus/rep_set/seqs_rep_set.fasta ;

assign_taxonomy.py -m rdp -c 0.80 -i otus/rep_set/seqs_rep_set.fasta -o otus/blast_assigned_taxonomy --id_to_taxonomy_fp ITS_lactobacilli/ITS_taxonomy.txt --reference_seqs_fp ITS_lactobacilli/ITS_names_rev.fas ;

make_otu_table.py -i otus/DeNovo_otus/seqs_otus.txt -t otus/blast_assigned_taxonomy/seqs_rep_set_tax_assignments.txt -o otus/otu_table_blast.biom ;

rm -fr temp/wf_taxa_summary_blast ; summarize_taxa_through_plots.py -i otus/otu_table_blast.biom -o temp/wf_taxa_summary_blast -m Fasting_Map_fw.txt &>/dev/null ;

cp temp/wf_taxa_summary_blast/otu_table_blast_L6.txt output/taxonomy_output_blast.txt ;



###NORMALIZATION OF THE RESULTS BY rRNA LOCI NUMBER###

mkdir tempnorm ;
cp output/taxonomy_output_blast.txt tempnorm/taxonomy_output_blast.txt ;
grep -v "#" tempnorm/taxonomy_output_blast.txt > tempnorm/taxonomy_output_blast2.txt ;

cut -f 1 tempnorm/taxonomy_output_blast.txt > tempnorm/complete_taxonomy_headed.txt ;
cut -f 2 tempnorm/taxonomy_output_blast.txt > tempnorm/relative_abundance_headed.txt ;
cut -f 1 tempnorm/taxonomy_output_blast2.txt > tempnorm/complete_taxonomy.txt ;
cut -f 2 tempnorm/taxonomy_output_blast2.txt > tempnorm/relative_abundance.txt ;

sed -i 's/;/\t/g' tempnorm/taxonomy_output_blast2.txt ;

cut -f 6 tempnorm/taxonomy_output_blast2.txt > tempnorm/species.txt ;

cp ITS_lactobacilli/rRNAloci_average_Qiime_wo_zero.txt tempnorm/rRNAloci_average_Qiime_wo_zero.txt ;
echo "Other	5.61321" >> tempnorm/rRNAloci_average_Qiime_wo_zero.txt ;
awk -f software/vlookup.awk tempnorm/rRNAloci_average_Qiime_wo_zero.txt tempnorm/species.txt > tempnorm/species_lookedup.txt ;

sed -i 's/ /\t/g' tempnorm/species_lookedup.txt ;
cut -f 2 tempnorm/species_lookedup.txt > tempnorm/column_number_rRNA_loci.txt ;


paste tempnorm/relative_abundance.txt tempnorm/column_number_rRNA_loci.txt > tempnorm/to_division.txt ;


awk -v OFS='\t' '{printf "%s\t%s\t%0.20f\n", $1 , $2 , $1/$2}' tempnorm/to_division.txt > tempnorm/divided.txt ;
cut -f 3 tempnorm/divided.txt > tempnorm/column_divided.txt ;

awk -F '\t' '{sum += $1} END {printf "%.20f\n", sum}' tempnorm/column_divided.txt > tempnorm/column_divided_summed.txt ;

SUMMED=$(cat tempnorm/column_divided_summed.txt) ;

echo "Normalized_Results" > tempnorm/column_normalized_cycle_pre.txt ;
echo "Normalized_Results" >> tempnorm/column_normalized_cycle_pre.txt ;


sed "s/$/\t$SUMMED/g" tempnorm/column_divided.txt > tempnorm/column_divided2.txt ;
awk -v OFS='\t' '{printf "%s\t%s\t%0.20f\n", $1 , $2 , $1/$2}' tempnorm/column_divided2.txt > tempnorm/column_divided3.txt ;
cut -f 3 tempnorm/column_divided3.txt > tempnorm/column_divided4.txt ;

cat tempnorm/column_normalized_cycle_pre.txt tempnorm/column_divided4.txt > tempnorm/column_normalized_cycle.txt ;


paste output/taxonomy_output_blast.txt tempnorm/column_normalized_cycle.txt > output/taxonomy_output_blast_normalized_by_rRNA_loci_number.txt ;



####EXTRACT ITS OTUS 100%

mkdir temp/ITS_otu_extraction

#edit biom
biom convert -i otus/otu_table_blast.biom -o temp/ITS_otu_extraction/table_from_biom_da_ordinare.txt --to-tsv
cp otus/rep_set/seqs_rep_set.fasta temp/ITS_otu_extraction/. ;
cp otus/blast_assigned_taxonomy/seqs_rep_set_tax_assignments.txt temp/ITS_otu_extraction/. ;
tail -n +3 temp/ITS_otu_extraction/table_from_biom_da_ordinare.txt > temp/ITS_otu_extraction/table_from_biom_senza_intestazione.txt ;
cat temp/ITS_otu_extraction/table_from_biom_da_ordinare.txt | head -n 2 > temp/ITS_otu_extraction/table_from_biom_intestazione.txt ;
sort -k1,1nr temp/ITS_otu_extraction/table_from_biom_senza_intestazione.txt > temp/ITS_otu_extraction/table_from_biom_ordinato.txt ;

#edit assignments
cut -f1,2 temp/ITS_otu_extraction/seqs_rep_set_tax_assignments.txt > temp/ITS_otu_extraction/seqs_rep_set_tax_assignments_da_ordinare.txt
sort -k1,1nr temp/ITS_otu_extraction/seqs_rep_set_tax_assignments_da_ordinare.txt > temp/ITS_otu_extraction/seqs_rep_set_tax_assignments_ordinato.txt ;

#edit sequences
grep ">" temp/ITS_otu_extraction/seqs_rep_set.fasta > temp/ITS_otu_extraction/seqs_rep_set_colonna1.fasta ;
cat temp/ITS_otu_extraction/seqs_rep_set_colonna1.fasta | awk 'BEGIN { FS = " " } ; { print $1 }' > temp/ITS_otu_extraction/seqs_rep_set_colonna1_2.fasta ;
sed -i "s|>||" temp/ITS_otu_extraction/seqs_rep_set_colonna1_2.fasta ;
grep -v ">" temp/ITS_otu_extraction/seqs_rep_set.fasta > temp/ITS_otu_extraction/seqs_rep_set_colonna2.fasta
paste temp/ITS_otu_extraction/seqs_rep_set_colonna1_2.fasta temp/ITS_otu_extraction/seqs_rep_set_colonna2.fasta > temp/ITS_otu_extraction/seqs_rep_set_da_ordinare.fasta ;
sort -k1,1nr temp/ITS_otu_extraction/seqs_rep_set_da_ordinare.fasta > temp/ITS_otu_extraction/seqs_rep_set_ordinato.fasta ;
awk '{print ">"$0}' temp/ITS_otu_extraction/seqs_rep_set_ordinato.fasta > temp/ITS_otu_extraction/seqs_rep_set_ordinato_2.fasta ;

#paste two files
paste temp/ITS_otu_extraction/seqs_rep_set_tax_assignments_ordinato.txt temp/ITS_otu_extraction/seqs_rep_set_ordinato_2.fasta > temp/ITS_otu_extraction/paste_primi_due_file.txt ;
cut -f1,2,4 temp/ITS_otu_extraction/paste_primi_due_file.txt > temp/ITS_otu_extraction/paste_primi_due_file_senza3.txt ;
sed -i "s|\t|XXXXXX|" temp/ITS_otu_extraction/paste_primi_due_file_senza3.txt ;

#rows selection
cut -f1 temp/ITS_otu_extraction/table_from_biom_ordinato.txt > temp/ITS_otu_extraction/table_from_biom_ordinato_primacolonna.txt ;
while read NUMSEQ1
do
grep "^${NUMSEQ1}XXXXXX" temp/ITS_otu_extraction/paste_primi_due_file_senza3.txt >> temp/ITS_otu_extraction/paste_tre_file.txt
done < temp/ITS_otu_extraction/table_from_biom_ordinato_primacolonna.txt
sed -i "s|XXXXXX|\t|g" temp/ITS_otu_extraction/paste_tre_file.txt ;
sed -i "s|^|OTU_ID\ttaxonomy\tsequence\t|g" temp/ITS_otu_extraction/table_from_biom_intestazione.txt ;

#paste three files
paste temp/ITS_otu_extraction/paste_tre_file.txt temp/ITS_otu_extraction/table_from_biom_ordinato.txt > temp/ITS_otu_extraction/pastate ;
cat temp/ITS_otu_extraction/table_from_biom_intestazione.txt temp/ITS_otu_extraction/pastate > temp/ITS_otu_extraction/tabulato.tbl ;
cut -f4 --complement temp/ITS_otu_extraction/tabulato.tbl > output/OTUs_table_taxonomy.txt ;

#####END OF OTUs EXTRACTION#####


rm -fr temp ;
rm -fr fastq ;


